#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <limits.h>

#include "common.h"
#include "macros.h"
#include "xmalloc.h"
#include "vm.h"
#include "task.h"

struct task *task_new(task_mode_t mode, const char *name)
{
	struct task *task;

	task = xmalloc0(sizeof(*task));

	task->mode = mode;
	xstrncpy(task->name, name, sizeof(task->name));

	return task;
}

void task_free(struct task **task)
{
	return_if_fail(task != NULL);
	return_if_fail((*task) != NULL);

	xfree((*task));
	*task = NULL;
}

void task_get_ctx(struct vm *machine, struct task *task)
{
	return_if_fail(machine != NULL);
	return_if_fail(task != NULL);

	task->mmap_begin = machine->mmap_begin;
	task->mmap_size = machine->mmap_size;
	task->pc = machine->pc;
	task->sp = machine->sp;
	task->fp = machine->fp;
	task->start_addr = machine->start_addr;
	task->stop_addr = machine->stop_addr;
	task->stack = machine->stack;
	task->stack_size = machine->stack_size;
	task->text = machine->text;
	task->text_size = machine->text_size;
	task->init = machine->init;
	task->init_size = machine->init_size;
	task->data = machine->data;
	task->data_size = machine->data_size;
	task->ro = machine->ro;
	task->ro_size = machine->ro_size;
	task->string = machine->string;
	task->sym_tb = machine->sym_tb;
	task->sym_tb_size = machine->sym_tb_size;
}

void task_set_ctx(struct vm *machine, struct task *task)
{
	return_if_fail(machine != NULL);
	return_if_fail(task != NULL);

	machine->mmap_begin = task->mmap_begin;
	machine->mmap_size = task->mmap_size;

	if (task->pc == task->stop_addr)
		machine->pc = task->start_addr;
	else
		machine->pc = task->pc;

	machine->sp = task->sp;
	machine->fp = task->fp;
	machine->start_addr = task->start_addr;
	machine->stop_addr = task->stop_addr;
	machine->stack = task->stack;
	machine->stack_size = task->stack_size;
	machine->text = task->text;
	machine->text_size = task->text_size;
	machine->init = task->init;
	machine->init_size = task->init_size;
	machine->data = task->data;
	machine->data_size = task->data_size;
	machine->ro = task->ro;
	machine->ro_size = task->ro_size;
	machine->string = task->string;
	machine->sym_tb = task->sym_tb;
	machine->sym_tb_size = task->sym_tb_size;
}

void task_switch_ctx(struct vm *machine, struct task *old_task, struct task *new_task)
{
	return_if_fail(machine != NULL);
	return_if_fail(old_task != NULL);
	return_if_fail(new_task != NULL);

	task_get_ctx(machine, old_task);
	task_set_ctx(machine, new_task);
}
